math.randomseed(os.time())

require 'map'
require 'rays'

function dump(o)
    if type(o) == 'table' then
       local s = '{ '
       for k,v in pairs(o) do
          if type(k) ~= 'number' then k = '"'..k..'"' end
          s = s .. '['..k..'] = ' .. dump(v) .. ','
       end
       return s .. '} '
    else
       return tostring(o)
    end
 end

function love.load()
    -- Math
    sin = math.sin
    cos = math.cos
    -- Values
    player = {
        x = 400,
        y = 300,
        z = 0,
        r = 15,
        w = 15,
        h = 15,
        d = 15,
        dir = 90,
        velocity = 0,
        jump = false,
        falling = false,
        remembered_z = 0,
        sin = 0,
        cos = 0,
        sp = 0,
        init_sp = 2,
        jump_force = 2,
    }
    trying_to_exit = false
    exit_timer = 0
    rays = {
        [1] = {}, -- distances
        [2] = {}, -- xs
        [3] = {}, -- ys
        [4] = {}, -- objects
    }
    raycast = {
        RES = 0.5,
        FOV = 80,
    }
    mouse = {
        x = 0,
        y = 0,
        locked = false,
        sensitivity = 3
    }
    current_map = maps.map1
    brightness = 3
    vx = 0
    vy = 0
    vz = 0
    collided = false
end

function love.update(dt)
    player.remembered_z = player.z
    if trying_to_exit then
        exit_timer = exit_timer + dt
    end
    if exit_timer > 2 then
        exit_timer = 0
        trying_to_exit = false
    end
    player.sp = player.init_sp + dt * 10
    mouse.x, mouse.y = love.mouse.getPosition()
    love.mouse.setVisible(true)
    if mouse.locked then
        love.mouse.setVisible(false)
        love.mouse.setX(love.graphics.getWidth() / 2)
        love.mouse.setY(love.graphics.getHeight() / 2)
        player.dir = player.dir - (mouse.x - love.graphics.getWidth() / 2) / 10 * mouse.sensitivity
    end

    player.sin = sin(math.rad(player.dir))
    player.cos = cos(math.rad(player.dir))

    if (love.keyboard.isDown("w")) or (love.keyboard.isDown("up")) then
        player.x = player.x + player.cos * player.sp
        player.y = player.y - player.sin * player.sp
    end
    if (love.keyboard.isDown("s")) or (love.keyboard.isDown("down")) then
        player.x = player.x - player.cos * player.sp
        player.y = player.y + player.sin * player.sp
    end
    if (love.keyboard.isDown("a")) or (love.keyboard.isDown("left")) then
        player.x = player.x - player.sin * player.sp
        player.y = player.y - player.cos * player.sp
    end
    if (love.keyboard.isDown("d")) or (love.keyboard.isDown("right")) then
        player.x = player.x + player.sin * player.sp
        player.y = player.y + player.cos * player.sp
    end
    -- [[
    
    for k, v in pairs(current_map) do
        collided, vx, vy, vz = collide(player.x - 7.5, player.y - 7.5, player.z, player.w, player.h, player.d, v.x, v.y, v.z, v.w, v.h, v.d)
        player.x = player.x + vx
        player.y = player.y + vy
        player.z = player.z
    end
    

    for k, v in pairs(rays) do
        v = nil
    end
    table.insert(rays, 1, {})
    table.insert(rays, 2, {})
    table.insert(rays, 3, {})
    table.insert(rays, 4, {})
    Ray_Plain(player.x, player.y, player.dir, player.dir, raycast.RES * 2, current_map, rays[1], rays[2], rays[3], rays[4], raycast.RES, raycast.FOV)
end

function love.keypressed(button)
    if button == "m" then
        mouse.locked = not mouse.locked
    end
    if button == "escape" then
        if not trying_to_exit then
            trying_to_exit = true
        else
            love.event.quit()
        end
    end
end

function love.draw()
    love.graphics.setColor(1,1,1,1)
    -- [[2d graphics
    for i, v in pairs(current_map) do
        love.graphics.rectangle("fill", v.x, v.y, v.w, v.h)
    end
    love.graphics.circle("fill", player.x, player.y, player.r)--]]
    for k,v in pairs(rays[1]) do
        for i = 1, #rays[1][k] do
            love.graphics.line(player.x, player.y, rays[2][k][i], rays[3][k][i])
        end
    end--]]
    -- [[3d graphics
    local gap = love.graphics.getWidth() / #rays[1]
    if rays[1][1] ~= nil then
        for k,d in pairs(rays[1]) do -- Each ray
            for i,v in pairs(rays[1][k]) do -- Each pos in that ray
                if rays[1][k][#rays[1][k] - i + 1] ~= nil then
                    love.graphics.setColor(0 + (brightness * 1000 / rays[1][k][#rays[1][k] - i + 1] / (500 / brightness)) * rays[4][k][#rays[4][k] - i + 1].r ,0 + (brightness * 1000 / rays[1][k][#rays[1][k] - i + 1] / (500 / brightness)) * rays[4][k][#rays[4][k] - i + 1].g ,0 + (brightness * 1000 / rays[1][k][#rays[1][k] - i + 1] / (500 / brightness)) * rays[4][k][#rays[4][k] - i + 1].b)
                    love.graphics.rectangle("fill", 0 - gap + (gap * k), (love.graphics.getHeight() / 2) - ((400000 / raycast.FOV) / rays[1][k][#rays[1][k] - i + 1]) + (player.z / rays[1][k][#rays[1][k] - i + 1] * 1000) - (rays[4][k][#rays[4][k] - i + 1].z / rays[1][k][#rays[1][k] - i + 1] * 180) - (rays[4][k][#rays[4][k] - i + 1].d / rays[1][k][#rays[1][k] - i + 1] / 18 * 2000) - 25, gap, ((400000 / raycast.FOV) / rays[1][k][#rays[1][k] - i + 1]) * 2 + (rays[4][k][#rays[4][k] - i + 1].d / rays[1][k][#rays[1][k] - i + 1] * 180) * 2)
                    love.graphics.setColor(1,1,1,1)
                end
            end
        end
    end--]]
    love.graphics.setColor(1,1,1,1)
    love.graphics.printf("FPS: "..love.timer.getFPS().."\n"..tostring(player.jump).."\t"..tostring(player.velocity).."\t"..tostring(player.z).."\n".."Press M to lock/unlock mouse", 0, 0, love.graphics.getWidth(), "center")
    if trying_to_exit then
        love.graphics.setColor(1,1,1,1 - (exit_timer / 2))
        love.graphics.print("Press ESC again to exit")
    end
    love.graphics.setColor(1,1,1,1)
end